package net.minecraft.src;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

public class EntityNameplate extends Entity
{
    public EntityLiving entityOwner;
    public String petName;
    public String ownerClass;

    public static EntityNameplate CreateNameplate(World world, EntityLiving owner)
    {
        if(! mod_Nameplate.EntityNameable(owner)) {
            return null;
        }

        if(mod_Nameplate.isLMMEnable && owner.getEntityString() == "LittleMaid") {
            // メイド用
            return new EntityNameplateMaid(world, owner);
        }
        return new EntityNameplate(world, owner);
    }

    public EntityNameplate(World world)
    {
        super(world);
    }

    public EntityNameplate(World world, EntityLiving owner)
    {
        super(world);
        System.out.println(String.format("EntityNameplate, owner = %s", new Object[] { owner.getEntityString() }));
        this.petName = "";

        if (owner != null)
        {
            this.setOwnerEntity(owner);
            this.setPosition(this.entityOwner.posX, this.entityOwner.posY, this.entityOwner.posZ);
            System.out.println(String.format("new nameplate, entity id = %d", new Object[] {Integer.valueOf(this.entityOwner.entityId)}));
        }

        // ownerが命名可能じゃなかったときって、どうすんの？
    }

    public void entityInit()
    {
    }

    public void setDead()
    {
    	super.setDead();
    	mod_Nameplate.NameplateList.remove(this);
    }

    protected boolean setOwnerEntity(EntityLiving owner)
    {
        if(! mod_Nameplate.EntityNameable(owner)) {
            return false;
        }

        String var2 = owner.getEntityString();
        //System.out.println(String.format("Owner is %s.", new Object[] {var2}));
        this.entityOwner = owner;
        this.ownerClass = var2;
        mod_Nameplate.registerEntityNameplate(this);
        return true;
    }

    protected EntityLiving FindOwner()
    {
        
        TreeMap<Float, EntityLiving> candidates = SearchOwnerCandidated();
        if(candidates.size() == 0)
        {
            return null;
        }
        return candidates.firstEntry().getValue();
    }
    
    protected TreeMap<Float, EntityLiving> SearchOwnerCandidated()
    {
        System.out.println("Search nameplate owner...");
        // owner候補を探す
        TreeMap<Float, EntityLiving> candidates = new TreeMap<Float, EntityLiving>();
        double var2 = 0.5F;
        ArrayList var3 = new ArrayList();
        Iterator var4 = this.worldObj.loadedEntityList.iterator();
        Object var5;

        while (var4.hasNext())
        {
            var5 = var4.next();

            if (var5 instanceof EntityNameplate)
            {
                var3.add((EntityNameplate)var5);
            }
        }
        //System.out.println(String.format("var3.size() = %d", new Object[] { var3.size() }));

        var4 = this.worldObj.loadedEntityList.iterator();

        while (var4.hasNext())
        {
            var5 = var4.next();

            if(mod_Nameplate.EntityNameable(var5))
            {
                EntityLiving var6 = (EntityLiving)var5;
                boolean skip = false;
                Iterator var8 = var3.iterator();

                // すでにNameplateを持っているEntityは除く
                while(var8.hasNext()) {
                    EntityNameplate var9 = (EntityNameplate)var8.next();

                    if (var9.entityOwner != null && var9.entityOwner.entityId == var6.entityId)
                    {
                        skip = true;
                        break;
                    }
                }
                
                if(skip) {
                    continue;
                }

                float var10 = this.getDistanceToEntity(var6);
                //System.out.println(String.format("this.getDistanceToEntity(var6) = %f", new Object[] { var10 }));
                if(var10 <= var2)
                {
                    candidates.put(var10, (EntityLiving)var6);
                }
            }
        }

        System.out.println(String.format("Found %d candidates.", new Object[] { candidates.size() }));
        return candidates;
    }

    public void onEntityUpdate()
    {
        super.onEntityUpdate();

        // 最寄りのEntityを探す
        if (this.entityOwner == null)
        {
            if (!this.setOwnerEntity(this.FindOwner()))
            {
                System.out.println("Failed to setOwnerEntity.");
                this.setDead();
                return;
            }

            System.out.println(String.format("found nearest owner, owner id = %d", new Object[] {Integer.valueOf(this.entityOwner.entityId)}));
        }

        if (this.entityOwner.isDead)
        {
            this.setDead();
            System.out.println("Nameplate owner is dead.");
            mod_Nameplate.addChatMessageToEntityOwner(this.entityOwner, String.format("%s is dead.", new Object[] { this.petName }));
        }
        else
        {
            this.setPosition(this.entityOwner.posX, this.entityOwner.posY, this.entityOwner.posZ);
        }
    }

    public void writeEntityToNBT(NBTTagCompound nbttagcompound)
    {
        if (this.entityOwner != null)
        {
            nbttagcompound.setString("Name", this.petName);

            if (this.entityOwner != null)
            {
                nbttagcompound.setDouble("OwnerX", this.entityOwner.posX);
                nbttagcompound.setDouble("OwnerY", this.entityOwner.posY);
                nbttagcompound.setDouble("OwnerZ", this.entityOwner.posZ);
            }
            else
            {
                nbttagcompound.setDouble("OwnerX", this.posX);
                nbttagcompound.setDouble("OwnerY", this.posY);
                nbttagcompound.setDouble("OwnerZ", this.posZ);
            }
        }
    }
    protected void readEntityFromNBT(NBTTagCompound nbttagcompound)
    {
        //System.out.println("readEntityFromNBT");
        this.petName = nbttagcompound.getString("Name");
        this.posX = nbttagcompound.getDouble("OwnerX");
        this.posY = nbttagcompound.getDouble("OwnerY");
        this.posZ = nbttagcompound.getDouble("OwnerZ");
    }
}
